/*
 * Copyright (C) 2005-2009 Makoto Sato. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.objectfanatics.assertion.weaver.api.annotationprocessor;

import java.lang.annotation.Annotation;

import javassist.CtBehavior;
import javassist.CtMethod;
import jp.objectfanatics.assertion.constraints.ConstraintAnnotation;
import jp.objectfanatics.assertion.weaver.api.core.exception.IllegalUseOfConstraintAnnotationException;
import jp.objectfanatics.assertion.weaver.api.core.exception.IllegalUseOfParameterConstraintAnnotationException;
import jp.objectfanatics.assertion.weaver.api.core.exception.IllegalUseOfReturnValueConstraintAnnotationException;

//Checked 09.27.2009. Delete this line before release.
/**
 * Assertion weaver for a constraint annotation.
 * 
 * @author Makoto Sato
 */
public interface ConstraintAnnotationProcessor {

	/**
	 * Returns target constraint annotation. It must be annotated with {@link ConstraintAnnotation}.
	 * 
	 * @return target constraint annotation
	 */
	Class<? extends Annotation> getTargetConstraintAnnotation();

	// TODO: 下記２メソッドは、一見すると、戻り値がなくてもよいように思える。しかし、昔voidだったのをあえてbooleanにした経緯があったように記憶している（佐藤）。なので、実際の実装をみながら、検討しましょう。
	/**
	 * Weaves constraint assertion for a parameter.
	 * 
	 * @param ctBehavior
	 *            a handle for dealing with the target constructor or method
	 * @param paramIndex
	 *            target parameter index
	 * @param annotation
	 *            target annotation
	 * @return true if, and only if, assertion functions are woven into the class file bytes
	 * @throws IllegalUseOfParameterConstraintAnnotationException
	 */
	boolean processParameterConstraintAnnotation(CtBehavior ctBehavior, int paramIndex, javassist.bytecode.annotation.Annotation annotation) throws IllegalUseOfConstraintAnnotationException;

	/**
	 * Weaves constraint assertion for return value.
	 * 
	 * @param ctMethod
	 *            target method
	 * @param annotation
	 *            target annotation
	 * @return true if, and only if, assertion functions are woven into the class file bytes
	 * @throws IllegalUseOfReturnValueConstraintAnnotationException
	 */
	boolean processReturnValueConstraintAnnotation(CtMethod ctMethod, javassist.bytecode.annotation.Annotation annotation) throws IllegalUseOfReturnValueConstraintAnnotationException;
}
