/*
 * Copyright (C) 2005-2009 Makoto Sato. All Rights Reserved.
 * Copyright (C) 2010 OBJECT FANATICS Co.,Ltd. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.objectfanatics.assertion.weaver.api.core;

import jp.objectfanatics.assertion.weaver.api.core.exception.IllegalUseOfConstraintAnnotationException;
import jp.objectfanatics.assertion.weaver.api.core.exception.NoWeavingNeededException;
import jp.objectfanatics.assertion.weaver.api.core.exception.NotClassFileException;

//DOUBLE_CHECKED 03.31.2011
/**
 * Weaver for weaving assertion functions into a class file bytes.
 *
 * @author Makoto Sato
 * @author Hirokatsu Endo
 */
public interface Weaver {
    /**
     * Weaves assertion functions into a class file bytes.
     *
     * @param classFileBytes the class file bytes to be woven
     *
     * @return the woven class file bytes
     *
     * @throws NoWeavingNeededException
     * @throws NotClassFileException
     * @throws IllegalUseOfConstraintAnnotationException
     *
     * @throws IllegalArgumentException if <code>classFileBytes</code> is null
     */
    byte[] weave (byte[] classFileBytes) throws NoWeavingNeededException, NotClassFileException, IllegalUseOfConstraintAnnotationException;
}
