/*
 * Copyright (C) 2005-2009 Makoto Sato. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.objectfanatics.assertion.weaver.api.core.exception;

import jp.objectfanatics.assertion.weaver.api.core.exception.info.AnnotationInfo;
import jp.objectfanatics.assertion.weaver.api.core.exception.info.BehaviorInfo;

//DOUBLE_CHECKED 04.04.2010
/**
 * Signals that illegal use of constraint annotations is detected.
 * 
 * @author Makoto Sato
 */
abstract public class IllegalUseOfConstraintAnnotationException extends Exception {
	
	/**
	 * Returns short description.
	 * 
	 * @return short description
	 */
	abstract public String getDescription();
	
	/**
	 * Returns detailed description.
	 * 
	 * @return detailed description
	 */
	abstract public String getDetailedDescription();
	
	/**
	 * Returns target behavior information.
	 * 
	 * @return target behavior information
	 */
	abstract public BehaviorInfo getTargetBehavior();

	/**
	 * Returns target annotation information.
	 * 
	 * @return target annotation information
	 */
	public abstract AnnotationInfo getTargetAnnotation();
}
