/*
 * Copyright (C) 2005-2009 Makoto Sato. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.objectfanatics.assertion.weaver.api.core.exception.info;

import jp.objectfanatics.assertion.weaver.api.core.exception.LineNumberNotAvailableException;
import jp.objectfanatics.assertion.weaver.api.core.exception.info.TypeInfo.PackageNotFoundException;

//DOUBLE_CHECKED 04.04.2010
/**
 * Behavior(constructor or method) information.
 * 
 * @author Makoto Sato
 */
public interface BehaviorInfo {
	
	/**
	 * Returns line number.
	 * 
	 * @return line number
	 * @throws LineNumberNotAvailableException
	 *             if line number is not available
	 */
	int getLineNumber() throws LineNumberNotAvailableException;
	
	// TODO: long name の説明と、実際の long name の文字列の例を書きましょう。
	/**
	 * Returns long name.
	 * 
	 * @return long name
	 */
	String getLongName();
	
	// TODO: 具体例を書きましょう。たとえば  com.objectfanatics.Foo.bar() にアノテーションをつけたとしたらどうなるかとか。
	/**
	 * Returns declaring type information.
	 * 
	 * @return declaring type information
	 */
	TypeInfo getDeclaringType();
	
	/**
	 * Syntactic sugar : this.getDeclaringType().getSourceFileName().
	 */
	String getSourceFileName();
	
	/**
	 * Syntactic sugar : this.getDeclaringType().getPackageName().
	 */
	String getPackageName() throws PackageNotFoundException;
}
