/*
 * Copyright (C) 2005-2009 Makoto Sato. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.objectfanatics.assertion.weaver.impl.annotationprocessor;

import javassist.CtBehavior;
import javassist.CtMethod;
import javassist.bytecode.annotation.Annotation;
import jp.objectfanatics.assertion.constraints.Max;
import jp.objectfanatics.assertion.weaver.api.annotationprocessor.ConstraintAnnotationProcessor;
import jp.objectfanatics.assertion.weaver.api.core.exception.IllegalUseOfConstraintAnnotationException;
import jp.objectfanatics.assertion.weaver.api.core.exception.IllegalUseOfReturnValueConstraintAnnotationException;
import jp.objectfanatics.assertion.weaver.impl.annotationprocessor.numbercomparing.AbstractNumberComparingAssertionWeaver;

/**
 * Implementation of {@link ConstraintAnnotationProcessor} for @{@link Max} annotation.
 * 
 * @author Makoto Sato
 */
public class MaxWeaverImpl extends AbstractNumberComparingAssertionWeaver {
	private static final String COMPARATIVE_OPERATOR = ">";
	private static final String COMPARATIVE_OPERATOR_DESCRIPTION = "greater";
	public MaxWeaverImpl() {
		super(Max.class, COMPARATIVE_OPERATOR, COMPARATIVE_OPERATOR_DESCRIPTION);
	}
	public void weaveMaxParameterConstraintAssertion(CtBehavior ctBehavior, int paramIndex, double max, Annotation annotation) throws IllegalUseOfConstraintAnnotationException {
		weaveNumberComparingParameterConstraintAssertion(ctBehavior, paramIndex, max, annotation);
	}
	public void weaveMaxReturnValueConstraintAssertion(CtMethod ctMethod, double max, Annotation annotation) throws IllegalUseOfReturnValueConstraintAnnotationException {
		weaveNumberComparingReturnValueConstraintAssertion(ctMethod, max, annotation);
	}
}
