package jp.objectfanatics.assertion.weaver.impl.annotationprocessor.numbercomparing.assertionconditiongenerator;

import javassist.CtBehavior;
import javassist.CtClass;
import jp.objectfanatics.assertion.weaver.impl.annotationprocessor.numbercomparing.AbstractNumberComparingAssertionWeaver;
import jp.objectfanatics.assertion.weaver.impl.annotationprocessor.numbercomparing.assertionconditiongenerator.ComparableNumberAssertionConditionExpressionGenerator.NotSupportedException;

public class ComparableNumberAssertionConditionExpressionGenerators {
	private final ComparableNumberAssertionConditionExpressionGenerator[] conditionExpressionGenerators;
	
	public ComparableNumberAssertionConditionExpressionGenerators(String comparativeOperator, Class<? extends java.lang.annotation.Annotation> targetConstraintAnnotation) {
		conditionExpressionGenerators = new ComparableNumberAssertionConditionExpressionGenerator[]{
			new ByteAssertionConditionExpressionGenerator(comparativeOperator, targetConstraintAnnotation),
			new ShortAssertionConditionExpressionGenerator(comparativeOperator, targetConstraintAnnotation),
			new IntegerAssertionConditionExpressionGenerator(comparativeOperator, targetConstraintAnnotation),
			new LongAssertionConditionExpressionGenerator(comparativeOperator, targetConstraintAnnotation),
			new FloatAssertionConditionExpressionGenerator(comparativeOperator, targetConstraintAnnotation),
			new BigIntegerAssertionConditionExpressionGenerator(comparativeOperator, targetConstraintAnnotation),
			new DoubleAssertionConditionExpressionGenerator(comparativeOperator, targetConstraintAnnotation),
			new BigDecimalAssertionConditionExpressionGenerator(comparativeOperator, targetConstraintAnnotation),
		};
	}
	
	public boolean isSupported(CtClass type) {
		for (ComparableNumberAssertionConditionExpressionGenerator comparableNumberAssertionConditionExpressionGenerator : conditionExpressionGenerators) {
			if (comparableNumberAssertionConditionExpressionGenerator.isSupported(type)) return true;
		}
		return false;
	}
	public String getAssertionErrorCondition(CtBehavior ctBehavior, CtClass type, String valueIdentifier, double threshold) throws AbstractNumberComparingAssertionWeaver.ThresholdOutOfBoundsException {
		for (ComparableNumberAssertionConditionExpressionGenerator conditionExpressionGenerator : conditionExpressionGenerators) {
			try {
				return conditionExpressionGenerator.getAssertionErrorConditionExpression(ctBehavior, type, valueIdentifier, threshold);
			} catch (NotSupportedException e) {
				// do nothing.
			}
		}
		throw new IllegalStateException("Unexpected type : " + type.getName());
	}
}
