package jp.objectfanatics.assertion.weaver.impl.annotationprocessor.numbercomparing.assertionconditiongenerator;

import javassist.CtBehavior;
import javassist.CtClass;
import jp.objectfanatics.assertion.weaver.impl.annotationprocessor.numbercomparing.AbstractNumberComparingAssertionWeaver.ThresholdOutOfBoundsException;

abstract public class ThresholdValidatableComparableNumberAssertionConditionExpressionGeneratorBase extends ComparableNumberAssertionConditionExpressionGeneratorBase {
	
	public ThresholdValidatableComparableNumberAssertionConditionExpressionGeneratorBase(String comparativeOperator, Class<? extends java.lang.annotation.Annotation> targetConstraintAnnotation) {
		super(comparativeOperator, targetConstraintAnnotation);
	}
	
	@Override
	protected final String getAssertionTargetValueExpression(CtBehavior ctBehavior, CtClass type, String valueIdentifier, double threshold) throws ThresholdOutOfBoundsException {
		validateThreshold(ctBehavior, threshold, type);
		return doGetAssertionTargetValueExpression(ctBehavior, type, valueIdentifier, threshold);
	}
	
	abstract protected String doGetAssertionTargetValueExpression(CtBehavior ctBehavior, CtClass type, String valueIdentifier, double threshold) throws ThresholdOutOfBoundsException;

	/**
	 * Validates the <code>threshold</code> to ensure it is in the proper range.
	 * 
	 * @param ctBehavior
	 *            {@link CtBehavior}
	 * @param threshold
	 * @throws InvalidAnnotationValueException
	 *             if the <code>threshold</code> is out of range.
	 */
	protected final void validateThreshold(CtBehavior ctBehavior, double threshold, CtClass type) throws ThresholdOutOfBoundsException {
		if (isLarger(threshold)) throw new ThresholdOutOfBoundsException(ctBehavior, threshold, true, getUpperLimitDesc(), type, targetConstraintAnnotation);
		if (isSmaller(threshold)) throw new ThresholdOutOfBoundsException(ctBehavior, threshold, false, getLowerLimitDesc(), type, targetConstraintAnnotation);
	}

	/**
	 * for template method {@link #validateThreshold(CtBehavior, double, CtClass)}
	 */
	abstract protected String getLowerLimitDesc();
	
	/**
	 * for template method {@link #validateThreshold(CtBehavior, double, CtClass)}
	 */
	abstract protected String getUpperLimitDesc();

	/**
	 * for template method {@link #validateThreshold(CtBehavior, double, CtClass)}
	 */
	abstract protected boolean isSmaller(double threshold);

	/**
	 * for template method {@link #validateThreshold(CtBehavior, double, CtClass)}
	 */
	abstract protected boolean isLarger(double threshold);
}
