/*
 * Copyright (C) 2005-2009 Makoto Sato. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.objectfanatics.assertion.weaver.impl.core.exception.info;

import static jp.objectfanatics.commons.javassist.JavassistUtils.getCtClass;
import javassist.ClassPool;
import javassist.bytecode.annotation.Annotation;
import jp.objectfanatics.assertion.weaver.api.core.exception.info.AnnotationInfo;
import jp.objectfanatics.assertion.weaver.api.core.exception.info.TypeInfo;

//Checked 09.26.2009. Delete this line before release.
/**
 * An implementation class of {@link AnnotationInfo}.
 * 
 * @author Makoto Sato
 */
public class AnnotationInfoImpl implements AnnotationInfo {
	
	private final TypeInfo type;
	
	/**
	 * Constructor.
	 * 
	 * @param classPool
	 *            {@link ClassPool}
	 * @param annotation
	 *            {@link Annotation}
	 * @throws IllegalArgumentException
	 *             if <code>classPool</code> is <code>null</code>
	 * @throws IllegalArgumentException
	 *             if <code>annotation</code> is <code>null</code>
	 */
	public AnnotationInfoImpl(ClassPool classPool, Annotation annotation) {
		if (classPool == null) throw new IllegalArgumentException("classPool is null.");
		if (annotation == null) throw new IllegalArgumentException("annotation is null.");
		this.type = new TypeInfoImpl(getCtClass(classPool, annotation));
	}
	
	@Override
	public TypeInfo getType() {
		return type;
	}
	
	@Override
	public String getName() {
		return type.getName();
	}
	
	@Override
	public String getSimpleName() {
		return type.getSimpleName();
	}
}
