/*
 * Copyright (C) 2005-2009 Makoto Sato. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.objectfanatics.assertion.weaver.impl.core.exception.info;

import javassist.CtBehavior;
import jp.objectfanatics.assertion.weaver.api.core.exception.LineNumberNotAvailableException;
import jp.objectfanatics.assertion.weaver.api.core.exception.info.BehaviorInfo;
import jp.objectfanatics.assertion.weaver.api.core.exception.info.TypeInfo;
import jp.objectfanatics.assertion.weaver.api.core.exception.info.TypeInfo.PackageNotFoundException;
import jp.objectfanatics.commons.javassist.JavassistUtils;

//Checked 09.26.2009. Delete this line before release.
/**
 * An implementation class of {@link BehaviorInfo}.
 * 
 * @author Makoto Sato
 */
abstract public class BehaviorInfoImpl implements BehaviorInfo {
	
	private final CtBehavior ctBehavior;
	
	/**
	 * Constructor.
	 * 
	 * @param ctBehavior
	 *            {@link CtBehavior}
	 * @throws IllegalArgumentException
	 *             if <code>ctBehavior</code> is <code>null</code>
	 */
	public BehaviorInfoImpl(CtBehavior ctBehavior) {
		if (ctBehavior == null) throw new IllegalArgumentException("ctBehavior is null.");
		this.ctBehavior = ctBehavior;
	}
	
	@Override
	public int getLineNumber() throws LineNumberNotAvailableException {
		try {
			return JavassistUtils.getLineNumber(ctBehavior);
		}
		catch (JavassistUtils.LineNumberNotAvailableException e) {
			throw new LineNumberNotAvailableException();
		}
	}
	
	@Override
	public String getLongName() {
		return ctBehavior.getLongName();
	}
	
	@Override
	public TypeInfo getDeclaringType() {
		return new TypeInfoImpl(ctBehavior.getDeclaringClass());
	}
	
	@Override
	public String getSourceFileName() {
		return getDeclaringType().getSourceFileName();
	}
	
	@Override
	public String getPackageName() throws PackageNotFoundException {
		return getDeclaringType().getPackageName();
	}
}
