/*
 * Copyright (C) 2005-2009 Makoto Sato. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.objectfanatics.assertion.weaver.impl.core.exception.info;

import static jp.objectfanatics.commons.javassist.JavassistUtils.getParameterNames;
import static jp.objectfanatics.commons.javassist.JavassistUtils.getParameterTypesWithoutContextParameter;
import javassist.CtBehavior;
import jp.objectfanatics.assertion.weaver.api.core.exception.info.ParameterInfo;
import jp.objectfanatics.assertion.weaver.api.core.exception.info.TypeInfo;
import jp.objectfanatics.commons.javassist.JavassistUtils.NoAvailableParameterNameInfoException;

//Checked 09.26.2009. Delete this line before release.
/**
 * An implementation class of {@link ParameterInfo}.
 * 
 * @author Makoto Sato
 */
public class ParameterInfoImpl implements ParameterInfo {
	
	private final int index;
	
	private final String name;
	
	private final TypeInfo type;
	
	/**
	 * Constructor.
	 * 
	 * @param index
	 *            index
	 * @param ctBehavior
	 *            {@link CtBehavior}
	 * @throws IllegalArgumentException
	 *             if <code>index</code> is negative number.
	 * @throws IllegalArgumentException
	 *             if <code>ctBehavior</code> is <code>null</code>
	 */
	public ParameterInfoImpl(int index, CtBehavior ctBehavior) {
		if (index < 0) throw new IllegalArgumentException("index must not be negative. index = " + index);
		if (ctBehavior == null) throw new IllegalArgumentException("ctBehavior is null.");
		this.index = index;
		this.type = new TypeInfoImpl(getParameterTypesWithoutContextParameter(ctBehavior)[index]);
		String nameTmp;
		try {
			nameTmp = getParameterNames(ctBehavior)[index];
		}
		catch (NoAvailableParameterNameInfoException e) {
			nameTmp = null;
		}
		this.name = nameTmp;
	}
	
	@Override
	public int getIndex() {
		return index;
	}
	
	@Override
	public String getName() throws ParameterNameNotAvailableException {
		if (name == null) throw new ParameterNameNotAvailableException();
		return name;
	}
	
	@Override
	public TypeInfo getType() {
		return type;
	}
}
